/* executable.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <gege2061@redaction-developpez.com>
 */

/**
 * Abstract class representing an program can launch by #ExecutableManager.
 */
public abstract class Valide.AbstractExecutable : Object
{
  /**
   * Executable current working directory, or NULL to inherit parent's.
   */
  protected abstract string working_directory
  {
    owned get;
  }

  /**
   * A message show when executable was launched.
   *
   * You can use \{$cmd\} for show the command.
   */
  protected abstract string start_msg
  {
    owned get;
  }

  /**
   * A message show when executable stop.
   */
  protected abstract string end_msg
  {
    owned get;
  }

  /**
   * An executable name.
   */
  protected abstract string executable
  {
    owned get;
    set;
  }

  /**
   * This function was called before adding line in ExecutableManager tree view.
   * You can modify it.
   *
   * @param sender : the object on which the signal is emitted.
   * @param line : the text.
   */
  protected virtual void parsing (ExecutableManager sender, ref string line)
  {
  }

  /**
   * This function wad called when line in ExecutableManager was activated.
   *
   * @param sender : the object on which the signal is emitted.
   * @param line : the line was actived.
   */
  protected virtual void line_activated (ExecutableManager sender, string line)
  {
  }

  private void cmd_start (ExecutableManager sender, string cmd)
  {
    string msg = "";
    try
    {
      Regex regex = new Regex ("\\{\\$cmd\\}",
                               RegexCompileFlags.OPTIMIZE,
                               RegexMatchFlags.NOTEMPTY);
      msg = regex.replace (this.start_msg, -1, 0, cmd,
                           RegexMatchFlags.NOTEMPTY);
    }
    catch (Error e)
    {
      debug (e.message);
      msg = this.start_msg;
    }
    sender.add_line (msg);
  }

  private void cmd_end (ExecutableManager sender, int status)
  {
    sender.cmd_start -= this.cmd_start;
    sender.cmd_end -= this.cmd_end;
    sender.add_line (this.end_msg);
  }

  /**
   * Launch executable.
   *
   * @param options : command line options.
   * @param dir : the current working directory. If NULL, use
   *              #AbstractExecutable.working_directory.
   */
  public virtual void run (string options = "", string? dir = null)
  {
    ExecutableManager exec = ExecutableManager.get_instance ();

    string directory;
    if (dir != null)
    {
      directory = dir;
    }
    else
    {
      directory = (this.working_directory == "") ? null : this.working_directory;
    }
    exec.cmd_start += this.cmd_start;
    exec.cmd_end += this.cmd_end;
    exec.parsing += this.parsing;
    exec.line_activated += this.line_activated;
    exec.run (directory, this.executable + " " + options);
  }
}

