/* executable-manager.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <gege2061@redaction-developpez.com>
 */

public errordomain Valide.ExecutableError
{
  NO_EXECUTABLE
}

public class Valide.ExecutableManager : Gtk.HBox
{
  private static ExecutableManager instance = null;
  private Gtk.ListStore list_store;
  private Gtk.TreeView tree_view;
  private Pid pid;

  public signal void cmd_start (string cmd);
  public signal void cmd_end (int status);
  public signal void parsing (ref string line);
  public signal void line_activated (string line);

  private string text
  {
    owned get
    {
      string str = "";
      Gtk.TreeIter iter;

      if (this.list_store.get_iter_first (out iter) == true)
      {
        string val = null;

        do
        {
          this.list_store.get (iter, 0, &val, -1);
          str += val + "\n";
        } while (this.list_store.iter_next (ref iter));
      }
      return str;
    }
  }

  public bool running
  {
    private set;
    get;
    default = false;
  }

  private ExecutableManager ()
  {
  }

  [Callback]
  private void clear ()
  {
    this.list_store.clear ();
  }

  [Callback]
  private void copy ()
  {
    weak Gtk.Clipboard clipboard = Gtk.Clipboard.get (Gdk.SELECTION_CLIPBOARD);
    string str = this.text;

    if (str != "")
    {
      clipboard.set_text (str, -1);
    }
    else
    {
      warning ("Nothing to copy!");
    }
  }

  [Callback]
  private void copy_bbcode ()
  {
    weak Gtk.Clipboard clipboard = Gtk.Clipboard.get (Gdk.SELECTION_CLIPBOARD);
    string str = this.text;

    if (str != "")
    {
      HashTable<string, string> pattern = new HashTable<string, string> (str_hash, str_equal);
      pattern.insert ("<i>(.*)</i>", "[i]\1[/i]");
      pattern.insert ("<b>(.*)</b>", "[b]\1[/b]");
      pattern.insert ("<span background=\"(.*)\">(.*)</span>", "[color=\"\1\"]\2[/color]");
      pattern.insert ("<span foreground=\"(.*)\">(.*)</span>", "[color=\"\1\"]\2[/color]");

      foreach (string key in pattern.get_keys ())
      {
        try
        {
          Regex regex = new Regex (key,
                                   RegexCompileFlags.OPTIMIZE,
                                   RegexMatchFlags.NOTEMPTY);
          str = regex.replace (str, -1, 0, pattern.lookup (key), RegexMatchFlags.NOTEMPTY);
        }
        catch (Error e)
        {
          warning (e.message);
        }
      }

      clipboard.set_text (str, -1);
    }
    else
    {
      warning ("Nothing to copy!");
    }
  }

  [Callback]
  private void row_activated (Gtk.TreeView sender, Gtk.TreePath path)
  {
    string line;
    Gtk.TreeIter iter;

    sender.model.get_iter (out iter, path);
    sender.model.get (iter, 1, out line);
    this.line_activated (line);
  }

  construct
  {
    Gtk.VBox vbox = new Gtk.VBox (false, 0);
    this.pack_start (vbox, false, false, 0);

    Gtk.Button button = new Gtk.Button ();
    button.relief = Gtk.ReliefStyle.NONE;
    button.image = new Gtk.Image.from_stock (Gtk.STOCK_CLEAR, Gtk.IconSize.SMALL_TOOLBAR);
    button.clicked += this.clear;
    vbox.pack_start (button, false, false, 0);

    button = new Gtk.Button ();
    button.relief = Gtk.ReliefStyle.NONE;
    button.image = new Gtk.Image.from_stock (Gtk.STOCK_COPY, Gtk.IconSize.SMALL_TOOLBAR);
    button.clicked += this.copy;
    vbox.pack_start (button, false, false, 0);

/*    button = new Gtk.Button ();
    button.relief = Gtk.ReliefStyle.NONE;
    button.image = new Gtk.Image.from_file (Path.build_filename (Config.PACKAGE_PIXMAPS_DIR, "bbcode.png"));
    button.clicked += this.copy_bbcode;
    vbox.pack_start (button, false, false, 0);
    button.set_sensitive (false);*/

    Gtk.ScrolledWindow scrolled_window = new Gtk.ScrolledWindow (null, null);
    this.list_store = new Gtk.ListStore (2, typeof (string), typeof (string));
    this.tree_view = new Gtk.TreeView.with_model (this.list_store);
    this.tree_view.row_activated += this.row_activated;
    Gtk.CellRendererText render = new Gtk.CellRendererText ();
    Gtk.TreeViewColumn column = new Gtk.TreeViewColumn.with_attributes (_("Message"), render, "markup", 0, null);
    this.tree_view.append_column (column);
    this.tree_view.headers_visible = false;

    scrolled_window.set_policy (Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC);
    scrolled_window.add (this.tree_view);
    this.pack_start (scrolled_window, true, true, 0);
  }

  public static ExecutableManager get_instance ()
  {
    if (instance == null)
    {
      instance = new ExecutableManager ();
    }
    return instance;
  }

  private bool build_iofunc (IOChannel ioc, IOCondition cond)
  {
    bool ret = false;

    if ((cond & (IOCondition.IN | IOCondition.PRI)) != 0)
    {
      string msg;
      IOStatus rv;

      try
      {
        rv = ioc.read_line (out msg, null, null);
        if (msg != null)
        {
          this.add_line (msg);
        }
        if (rv == IOStatus.NORMAL || rv == IOStatus.AGAIN)
        {
          ret = true;
        }
      }
      catch (Error e)
      {
        warning (e.message);
      }
    }
    return ret;
  }

  private void cmd_watch (Pid pid, int status)
  {
    this.running = false;
    UIManager.get_instance ().get_action ("project-abort").sensitive = false;
    this.cmd_end (status);
  }

  public void run (string? dir, string cmd)
  {
    if (!this.running)
    {
      try
      {
        this.running = true;
        string cmd2;

        this.clear ();
        Regex regex = new Regex ("[ ]{2,}",
                                 RegexCompileFlags.OPTIMIZE,
                                 RegexMatchFlags.NOTEMPTY);
        cmd2 = regex.replace (cmd, -1, 0, " ", RegexMatchFlags.NOTEMPTY);

        string[] args;
        int std_out, std_err;

        Shell.parse_argv (cmd2, out args);
        this.cmd_start (cmd2);
        Process.spawn_async_with_pipes (dir, args, null,
                                        SpawnFlags.SEARCH_PATH | SpawnFlags.DO_NOT_REAP_CHILD,
                                        null, out this.pid, null, out std_out,
                                        out std_err);
        UIManager.get_instance ().get_action ("project-abort").sensitive = true;
        IOCondition cond = IOCondition.IN | IOCondition.PRI
                           | IOCondition.ERR | IOCondition.HUP
                           | IOCondition.NVAL;
        Utils.set_up_io_channel (std_out, cond, true, build_iofunc);
        Utils.set_up_io_channel (std_err, cond, true, build_iofunc);
        ChildWatch.add (this.pid, this.cmd_watch);
      }
      catch (Error e)
      {
        warning (e.message);
        this.running = false;
        this.cmd_end (-1);
      }
    }
    else
    {
      warning ("Already running!");
    }
  }

  public void stop () throws Error
  {
    if (this.running)
    {
      pid_kill (this.pid);
    }
    else
    {
      throw new ExecutableError.NO_EXECUTABLE (_("No executable launch!"));
    }
  }

  public void add_line (string line)
  {
    try
    {
      Regex regex = new Regex ("[\n\r]*$",
                               RegexCompileFlags.OPTIMIZE,
                               RegexMatchFlags.NOTEMPTY);

      string line2 = regex.replace (line, -1, 0, "", RegexMatchFlags.NOTEMPTY);

      if (line2.len () > 0)
      {
        string[] lines = line2.split ("\n");
        
        foreach (string l in lines)
        {
          string html = l;
          Gtk.TreeIter iter;

          this.parsing (ref html);

          this.list_store.append (out iter);
          this.list_store.set (iter, 0, html, 1, l);

          Gtk.TreePath path = this.list_store.get_path (iter);
          this.tree_view.scroll_to_cell (path, null, false, 0, 0);
        }
      }
    }
    catch (Error e)
    {
      warning (e.message);
    }
  }
}

