/* plugin-registrar.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <gege2061@redaction-developpez.com>
 */

errordomain Valide.PluginError
{
  MODULE_LOADING,
  MODULE_VERSION
}

public class Valide.PluginRegistrar : Object
{
  private Type type;
  private Module module;
  private string module_path;
  private string _path;
  private PreferencePluginFunction preferences_plugin;
  public const string[] FILE_EXT = {"valide-plugin"};
  /** @bug vala doens't suport string array properties */
  public string[] authors;

  private delegate Type RegisterPluginFunction ();
  private delegate Gtk.Widget PreferencePluginFunction (Plugin self);

  public string path
  {
    construct
    {
      this._path = value;
      try
      {
        KeyFile key_file = new KeyFile ();
        key_file.load_from_file (value, KeyFileFlags.NONE);
        this.module_name = key_file.get_string ("Valide Plugin", "Module");
        this.vapi_version = key_file.get_integer ("Valide Plugin", "Vapi");
        this.name = key_file.get_string ("Valide Plugin", "Name");
        this.description = key_file.get_string ("Valide Plugin", "Description");
        this.icon_name = key_file.get_string ("Valide Plugin", "Icon");
        this.authors = key_file.get_string_list ("Valide Plugin", "Authors");
        this.website = key_file.get_string ("Valide Plugin", "Website");
        this.copyright = key_file.get_string ("Valide Plugin", "Copyright");
        this.register_plugin ();
        this.is_available = true;
      }
      catch (Error e)
      {
        debug (_("Can't load plugin '%s': %s"), value, e.message);
        this.description = e.message;
        this.name = Path.get_basename (Path.get_dirname (value));
        this.is_available = false;
      }
    }

    get
    {
      return this._path;
    }
  }

  private string module_name
  {
    get;
    private set;
  }

  private int vapi_version
  {
    get;
    private set;
  }

  public bool is_active
  {
    get
    {
      return (this.plugin != null);
    }
  }

  public bool is_available
  {
    get;
    private set;
  }

  public string name
  {
    get;
    private set;
  }

  public string description
  {
    get;
    private set;
  }

  public string icon_name
  {
    get;
    private set;
  }

  public string website
  {
    get;
    private set;
  }

  public string copyright
  {
    get;
    private set;
  }

  public bool is_configurable
  {
    get;
    private set;
  }

  public Plugin plugin
  {
    get;
    private set;
  }

  private void register_plugin () throws Error
  {
    this.module_path = Module.build_path (Path.get_dirname (this.path),
                                          this.module_name);

    if (this.vapi_version <= Config.VAPI_VERSION.to_int ())
    {
      this.module = Module.open (module_path, ModuleFlags.BIND_LAZY);
      if (this.module != null)
      {
        void* function;

        this.module.symbol ("register_plugin", out function);
        return_if_fail (function != null);
        RegisterPluginFunction register_plugin = (RegisterPluginFunction)function;

        this.type = register_plugin ();

        this.module.symbol ("create_configure_dialog", out function);
        this.preferences_plugin = (PreferencePluginFunction)function;
        this.is_configurable = (function != null);
      }
      else
      {
        this.is_available = false;
        throw new PluginError.MODULE_LOADING ("%s", module.error);
      }
    }
    else
    {
      this.is_available = false;
      throw new PluginError.MODULE_VERSION (_("Incompatible vapi version"));
    }
  }

  construct
  {
    assert (Module.supported ());
  }

  public PluginRegistrar (string filename)
  {
    this.path = filename;
  }

  ~PluginRegistrar ()
  {
    try
    {
      this.unload ();
      this.module = null;
    }
    catch (Error e)
    {
      debug (e.message);
    }
  }

  public void load () throws Error
  {
    if (this.is_available)
    {
      this.plugin = Object.new (this.type, "path", Path.get_dirname (this.module_path)) as Plugin;
    }
    else
    {
      throw new PluginError.MODULE_LOADING (_("Can't load module (%s): module is unavailable"),
                                            this.path);
    }
  }

  public void unload () throws Error
  {
    this.plugin = null;
  }

  public void preferences ()
  {
    if (this.is_available && this.is_active && this.is_configurable)
    {
      Gtk.Dialog dialog = new Gtk.Dialog.with_buttons ("Plugin preferences",
                                                       null,
                                                       Gtk.DialogFlags.MODAL,
                                                       Gtk.STOCK_CLOSE,
                                                       Gtk.ResponseType.CLOSE);
      dialog.vbox.pack_start (this.preferences_plugin (this.plugin),
                              true, true, 0);
      dialog.show_all ();
      dialog.run ();
      dialog.destroy ();
    }
  }
}

