/* utils.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 *  Nicolas Joseph <gege2061@redaction-developpez.com>
 */

public struct Valide.Position
{
  public int start_line;
  public int end_line;
  public int start_row;
  public int end_row;
}

public class Valide.Utils
{
  public static string str_replace (string str, string src, string dst) throws RegexError
  {
    Regex regex = new Regex ("(" + src + ")");
    return regex.replace (str, str.size (), 0, dst);
  }

  public static string get_relative_path (string filename, string path)
  {
    File parent = File.new_for_path (path);
    File descendant = File.new_for_path (filename);
    string relative_path = parent.get_relative_path (descendant);
    if (relative_path == null)
    {
      relative_path = filename;
    }
    return relative_path;
  }

  public static string get_extension (string filename)
  {
    string ext = filename.rchr (-1, '.');
    if (ext != null)
    {
      ext = ext.offset (1).down ();
    }
    return ext;
  }

  public static uint64 get_mtime (string filename) throws Error
  {
    File file = File.new_for_path (filename);
    FileInfo info = file.query_info (FILE_ATTRIBUTE_TIME_MODIFIED, 0, null);

    return info.get_attribute_uint64 (FILE_ATTRIBUTE_TIME_MODIFIED);
  }

  public static void set_up_io_channel (int fd, IOCondition cond,
                                        bool nblock, IOFunc func) throws IOChannelError
  {
    IOChannel ioc;

    ioc = new IOChannel.unix_new (fd);
    if (nblock)
    {
      try
      {
        ioc.set_flags (IOFlags.NONBLOCK);
      }
      catch (Error e)
      {
        debug (e.message);
      }
    }
    ioc.set_close_on_unref (true);
    ioc.add_watch (cond, func);
  }

  public static void chop (ref string s)
  {
    if (s.has_suffix ("\n"))
    {
      s = s.ndup (s.len () - 1);
    }
  }

  public static Gtk.Button create_small_button (Gtk.Image image)
  {
    Gtk.Button button = new Gtk.Button ();
    button.set_relief (Gtk.ReliefStyle.NONE);

    /* don't allow focus on the close button */
    button.set_focus_on_click (false);

    /* make it as small as possible */
    Gtk.RcStyle rcstyle = new Gtk.RcStyle ();
    rcstyle.xthickness = rcstyle.ythickness = 0;
    button.modify_style (rcstyle);

    image.show ();

    button.add (image);

    /* Set minimal size */
    button.style_set += (s) => {
      int h, w;

      Gtk.icon_size_lookup_for_settings (s.get_settings (), Gtk.IconSize.MENU,
                                         out w, out h);
      s.set_size_request (w + 2, h + 2);
    };

    return button;
  }

  public static Gtk.Button create_small_button_from_stock (string stock_id)
  {
    Gtk.Image image = new Gtk.Image.from_stock (stock_id, Gtk.IconSize.MENU);
    return Utils.create_small_button (image);
  }

  public static Gdk.Pixbuf get_pixbuf_for_stock (string stock_id, Gtk.IconSize size)
  {
    Gdk.Pixbuf pixbuf;

    Gtk.Invisible w = new Gtk.Invisible ();
    pixbuf = w.render_icon (stock_id, size, "vala");
    return pixbuf;
  }

  public static Gdk.Pixbuf get_pixbuf_for_theme (string name, Gtk.IconSize size) throws Error
  {
    Gdk.Pixbuf pixbuf;
    Gtk.IconTheme theme = Gtk.IconTheme.get_default ();
    weak Gtk.IconInfo info = theme.lookup_icon (name, size, 0);
    int width;

    Gtk.icon_size_lookup (size, out width, null);
    pixbuf = theme.load_icon (name, width, 0);
  
    return pixbuf;
  }

  public static string get_mime_type (string filename) throws Error
  {
    File file = File.new_for_path (filename);
    FileInfo file_info = file.query_info ("standard::*", FileQueryInfoFlags.NONE, null);
    return g_content_type_get_mime_type (file_info.get_content_type ());
  }
}

public class Valide.Markup
{
  private static void save (string filename, Xml.Doc* xml)
  {
    xml->save_file (filename);
  }

  public static void remove (string filename, string xpath)
  {
    Xml.Doc* xml = Xml.Parser.parse_file (filename);
    Xml.XPathContext xpath_context = new Xml.XPathContext (xml);
    Xml.XPathObject* xpath_obj = xpath_context.eval (xpath);
    for (int i = 0; i < xpath_obj->nodesetval->length (); i++)
    {
      xpath_obj->nodesetval->item (i)->unlink ();
    }
    Markup.save (filename, xml);
  }

  public static void add_child (string filename, string xpath, string name,
                                string? content=null)
  {
    Xml.Doc* xml = Xml.Parser.parse_file (filename);
    Xml.XPathContext xpath_context = new Xml.XPathContext (xml);
    Xml.XPathObject* xpath_obj = xpath_context.eval (xpath);
    xpath_obj->nodesetval->item (0)->new_text_child (null, name, content);
    Markup.save (filename, xml);
  }

  public static bool exist (string filename, string xpath)
  {
    bool ret = false;

    Xml.Doc* xml = Xml.Parser.parse_file (filename);
    Xml.XPathContext xpath_context = new Xml.XPathContext (xml);
    Xml.XPathObject* xpath_obj = xpath_context.eval (xpath);
    if (xpath_obj != null && xpath_obj->nodesetval->length () > 0)
    {
      ret = true;
    }
    return ret;
  }

  public static string get_content (string filename, string xpath)
  {
    string ret = null;
    Xml.Doc* xml = Xml.Parser.parse_file (filename);
    Xml.XPathContext xpath_context = new Xml.XPathContext (xml);
    Xml.XPathObject* xpath_obj = xpath_context.eval (xpath);
    if (xpath_obj != null && xpath_obj->nodesetval->length () > 0)
    {
      ret = xpath_obj->nodesetval->item (0)->content;
    }
    return ret;
  }

  public static Xml.NodeSet* get_nodes (string filename, string xpath)
  {
    Xml.Doc* xml = Xml.Parser.parse_file (filename);
    Xml.XPathContext xpath_context = new Xml.XPathContext (xml);
    Xml.XPathObject* xpath_obj = xpath_context.eval (xpath);
    return xpath_obj->nodesetval;
  }
}

public class Valide.Vala
{
  public string compiler_name
  {
    owned get
    {
      return "valac";
    }
  }

  public string prefix
  {
    owned get
    {
      string vala_path = Environment.find_program_in_path (this.compiler_name);
      string bin_path = Path.get_dirname (vala_path);

      return Path.build_filename (bin_path, "..");
    }
  }

  public string vapi_dir
  {
    owned get
    {
      string dir;

      if (Config.OS == "win32")
      {
        dir = Path.build_filename ("C:", "Documents and Settings", "All Users", "Application Data", "vala", "vapi");
      }
      else
      {
        dir = Path.build_filename (this.prefix, "share", "vala", "vapi");
      }
      return dir;
    }
  }

  public static List<string> get_package_list (List<string>? vapi_dir)
  {
    List<string> vapi_files = null;
    List<string> vapidir = null;

    if (vapi_dir != null)
    {
      foreach (string v in vapi_dir)
      {
        vapidir.append (v);
      }
    }

    /** @bug static property doesn't work */
    Vala vala = new Vala ();
    vapidir.append (vala.vapi_dir);

    foreach (string d in Environment.get_system_data_dirs ())
    {
      if (FileUtils.test (d, FileTest.IS_DIR))
      {
        vapidir.append (d);
      }
    }

    foreach (string d in vapidir)
    {
      try
      {
        string filename;
        Dir dir = Dir.open (d);

        while ((filename = dir.read_name ()) != null)
        {
          if (filename.has_suffix (".vapi"))
          {
            vapi_files.append (filename.substring (0, filename.size () - 5));
          }
        }
      }
      catch (Error e)
      {
        debug (e.message);
      }
    }
    return vapi_files;
  }
}

