/* window.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <gege2061@redaction-developpez.com>
 */

public class Valide.Window : Gtk.Window
{
  private static Window instance = null;
  private UIManager ui_manager;
  private ConfigManager config_manager;
  private DocumentManager documents;
  private ProjectManager projects;
  private ExecutableManager executables;
  private PluginManager plugins;
  private Gtk.WindowGroup owned_group;
  public Panel side_panel;
  public Panel bottom_panel;
  public Statusbar statusbar;

  [Callback]
  private bool on_delete_event ()
  {
    try
    {
      this.executables.stop ();
    }
    catch (Error e)
    {
      debug (e.message);
    }
    return !this.documents.close_all ();
  }

  private void message (string? log_domain, LogLevelFlags log_levels,
                        string message)
  {
    Gtk.MessageType type;

    switch (log_levels)
    {
      case LogLevelFlags.LEVEL_MESSAGE:
        type = Gtk.MessageType.INFO;
      break;
      case LogLevelFlags.LEVEL_WARNING:
        type = Gtk.MessageType.WARNING;
      break;
      case LogLevelFlags.LEVEL_CRITICAL:
        type = Gtk.MessageType.ERROR;
      break;
      default:
        type = Gtk.MessageType.OTHER;
      break;
    }

    Gtk.Dialog dialog = new Gtk.MessageDialog (null, Gtk.DialogFlags.MODAL, type,
                                               Gtk.ButtonsType.OK, message);
    dialog.run ();
    dialog.destroy ();
  }

	private void setup_ui_manager ()
	{
    this.ui_manager = UIManager.get_instance ();
    this.ui_manager.post_activate += this.on_action_activated;
    this.add_accel_group (this.ui_manager.get_accel_group ());
  }

  private void populate_window ()
  {
    Gtk.VBox main_box = new Gtk.VBox (false, 0);
    this.add (main_box);

		RecentManagerUI.get_instance ().update_file (this.ui_manager);
		RecentManagerUI.get_instance ().update_project (this.ui_manager);

    main_box.pack_start (this.ui_manager.get_widget ("/menubar"), false, true, 0);
    main_box.pack_start (this.ui_manager.get_widget ("/toolbar"), false, true, 0);

    Gtk.VPaned vpaned = new Gtk.VPaned ();
    main_box.pack_start (vpaned, true, true, 0);

    this.bottom_panel = new Panel (Gtk.Orientation.HORIZONTAL);
    this.bottom_panel.add_item_with_stock_icon (this.executables, "Executable",
                                                Gtk.STOCK_EXECUTE);
    this.bottom_panel.hide += () => {
      Gtk.ToggleAction action = (Gtk.ToggleAction)UIManager.get_instance ().get_action ("view-bottom-panel");
      action.set_active (false);
    };

    vpaned.pack2 (this.bottom_panel, false, false);

    Gtk.HPaned hpaned = new Gtk.HPaned ();
    vpaned.add1 (hpaned);

    this.side_panel = new Panel (Gtk.Orientation.VERTICAL);
    this.side_panel.add_item (this.projects, "Project",
                              new Gtk.Image.from_file (Path.build_filename (Config.PACKAGE_PIXMAPS_DIR, "icone-16.xpm")));
    this.side_panel.hide += () => {
      Gtk.ToggleAction action = (Gtk.ToggleAction)UIManager.get_instance ().get_action ("view-side-panel");
      action.set_active (false);
    };

    hpaned.add1 (this.side_panel);
    hpaned.add2 (this.documents);

    main_box.pack_start (this.statusbar, false, true, 0);
  }

  [Callback]
  private void on_action_activated (Gtk.UIManager sender, Gtk.Action action)
  {
    try
    {
      /* @bug http://bugzilla.gnome.org/show_bug.cgi?id=559644 */
      string action_name = action.name;
      switch (action_name)
      {
        /* File menu */
        case "file-new":
          if (this.projects.current != null)
          {
            this.projects.current.create_file ();
          }
          else
          {
            this.documents.create ();
          }
        break;
        case "file-open":
          this.documents.open ();
        break;
        case "file-save":
          if (this.documents.current != null)
          {
            this.documents.current.save ();
          }
          else
          {
            throw new DocumentError.NO_DOCUMENT (_("No document open!"));
          }
        break;
        case "file-save-as":
          if (this.documents.current != null)
          {
            this.documents.current.save_as ();
          }
          else
          {
            throw new DocumentError.NO_DOCUMENT (_("No document open!"));
          }
        break;
        case "file-close":
          if (this.documents.current != null)
          {
            this.documents.current.close ();
          }
          else
          {
            throw new DocumentError.NO_DOCUMENT (_("No document open!"));
          }
        break;
        case "file-close-all":
          if (this.documents.current != null)
          {
            this.documents.close_all ();
          }
          else
          {
            throw new DocumentError.NO_DOCUMENT (_("No document open!"));
          }
        break;
        case "quit":
          if (!this.on_delete_event ())
            this.destroy ();
        break;

        /* Edit menu */
        case "edit-undo":
          Gtk.bindings_activate (this.get_focus (), Gdk.KeySyms.z,
                                 Gdk.ModifierType.CONTROL_MASK);
        break;
        case "edit-redo":
          /* @bug No key binding found */
/*
          Gtk.bindings_activate (this.get_focus (), Gdk.KeySyms.z,
                                 Gdk.ModifierType.CONTROL_MASK
                                 | Gdk.ModifierType.SHIFT_MASK);
*/
          if (this.documents.current != null)
          {
            this.documents.current.redo ();
          }
          else
          {
            throw new DocumentError.NO_DOCUMENT (_("No document open!"));
          }
        break;
        case "edit-copy":
          Gtk.bindings_activate (this.get_focus (), Gdk.KeySyms.c,
                                 Gdk.ModifierType.CONTROL_MASK);
        break;
        case "edit-cut":
          Gtk.bindings_activate (this.get_focus (), Gdk.KeySyms.x,
                                 Gdk.ModifierType.CONTROL_MASK);
        break;
        case "edit-paste":
          Gtk.bindings_activate (this.get_focus (), Gdk.KeySyms.v,
                                 Gdk.ModifierType.CONTROL_MASK);
        break;
        case "edit-search":
          if (this.documents.current != null)
          {
            this.documents.current.search ();
          }
          else
          {
            throw new DocumentError.NO_DOCUMENT (_("No document open!"));
          }
        break;
        case "edit-find-next":
          if (this.documents.current != null)
          {
            this.documents.current.find_next ();
          }
          else
          {
            throw new DocumentError.NO_DOCUMENT (_("No document open!"));
          }
        break;
        case "edit-find-prev":
          if (this.documents.current != null)
          {
            this.documents.current.find_prev ();
          }
          else
          {
            throw new DocumentError.NO_DOCUMENT (_("No document open!"));
          }
        break;
        case "edit-goto-line":
          if (this.documents.current != null)
          {
            this.documents.current.goto_line ();
          }
          else
          {
            throw new DocumentError.NO_DOCUMENT (_("No document open!"));
          }
        break;
        case "edit-preferences":
          config_manager.preferences ();
        break;

        /* View menu */
        case "view-start-page":
          Gtk.ToggleAction toggle_action = (Gtk.ToggleAction)action;
          if (toggle_action.active)
          {
            this.documents.show_start_page ();
          }
          else
          {
            this.documents.hide_start_page ();
          }
        break;
        case "view-side-panel":
          Gtk.ToggleAction toggle_action = (Gtk.ToggleAction)action;
          if (toggle_action.active)
          {
            this.side_panel.show ();
          }
          else
          {
            this.side_panel.hide ();
          }
        break;
        case "view-bottom-panel":
          Gtk.ToggleAction toggle_action = (Gtk.ToggleAction)action;
          if (toggle_action.active)
          {
            this.bottom_panel.show ();
          }
          else
          {
            this.bottom_panel.hide ();
          }
        break;

        /* Project menu */
        case "project-new":
          this.projects.create ();
        break;
        case "project-open":
          this.projects.open ();
        break;
        case "project-add":
          if (this.projects.current != null)
          {
            this.projects.current.add_file ();
          }
          else
          {
            throw new ProjectError.NO_PROJECT (_("No project open!"));
          }
        break;
        case "project-remove":
          if (this.projects.current != null)
          {
            this.projects.current.remove_file ();
          }
          else
          {
            throw new ProjectError.NO_PROJECT (_("No project open!"));
          }
        break;
        case "project-options":
          if (this.projects.current != null)
          {
            this.projects.current.options ();
          }
          else
          {
            throw new ProjectError.NO_PROJECT (_("No project open!"));
          }
        break;
        case "project-compile":
          if (this.projects.current != null)
          {
            this.projects.current.compile ();
          }
          else
          {
            throw new ProjectError.NO_PROJECT (_("No project open!"));
          }
        break;
        case "project-execute":
          if (this.projects.current != null)
          {
            this.projects.current.execute ();
          }
          else
          {
            throw new ProjectError.NO_PROJECT (_("No project open!"));
          }
        break;
        case "project-abort":
          this.executables.stop ();
        break;
        case "project-close":
          if (this.projects.current != null)
          {
            this.projects.current.close ();
          }
          else
          {
            throw new ProjectError.NO_PROJECT (_("No project open!"));
          }
        break;

        /* Help menu */
        case "help-about":
          AboutDialog dialog = new AboutDialog ();
          dialog.run ();
          dialog.destroy ();
        break;

        /* Project popup */
        case "project-file-new":
          if (this.projects.current != null)
          {
            this.projects.current.create_file ();
          }
          else
          {
            throw new ProjectError.NO_PROJECT (_("No project open!"));
          }
        break;
        case "project-file-open":
          this.projects.open_selected_file ();
        break;
        case "project-file-add":
          if (this.projects.current != null)
          {
            this.projects.current.add_file ();
          }
          else
          {
            throw new ProjectError.NO_PROJECT (_("No project open!"));
          }
        break;
        case "project-file-remove":
          this.projects.remove_selected_file ();
        break;
      }
    }
    catch (Error e)
    {
      warning (e.message);
    }
  }

  [Callback]
  private static void on_active_tab_changed (Gtk.Notebook sender,
                                             Gtk.NotebookPage? page,
                                             uint page_num,
                                             Window self)
  {
    string title = "";
    iDocument document = self.documents.current;

    if (document != null)
    {
      if (document.path != "")
      {
        title += document.path + " - ";
      }
    }
    title += "Val(a)IDE";
    self.set_title (title);
  }

  private Window ()
  {
  }

  ~Window ()
  {
    this.plugins.free ();
  }

  construct
  {
    this.owned_group = new Gtk.WindowGroup ();
    this.group = this.owned_group;
    this.group.add_window (this);

    this.delete_event += this.on_delete_event;
    this.destroy += () => {
      Gtk.main_quit ();
    };

    this.window_state_event += (s, e) => {
	    if ((e.changed_mask &
	        (Gdk.WindowState.MAXIMIZED | Gdk.WindowState.FULLSCREEN)) != 0)
	    {
		    bool show;

		    show = !((e.new_window_state &
			    (Gdk.WindowState.MAXIMIZED | Gdk.WindowState.FULLSCREEN)) != 0);

		    this.statusbar.set_has_resize_grip (show);
	    }
    };

    this.config_manager = ConfigManager.get_instance ();
    this.documents = DocumentManager.get_instance ();
    Signal.connect_after (this.documents, "switch-page", (Callback)on_active_tab_changed, this);
    this.on_active_tab_changed (this.documents, null, 0, this);
    this.projects = ProjectManager.get_instance ();
    /** @note Circular dependancy with DocumentManager */
    this.executables = ExecutableManager.get_instance ();
    this.statusbar = Statusbar.get_instance ();
    this.setup_ui_manager ();
    this.populate_window ();
    this.maximize ();

    try
    {
      List<Gdk.Pixbuf> list;
      list.append (new Gdk.Pixbuf.from_file (Path.build_filename (Config.PACKAGE_PIXMAPS_DIR, "icone-16.xpm")));
      list.append (new Gdk.Pixbuf.from_file (Path.build_filename (Config.PACKAGE_PIXMAPS_DIR, "icone-32.xpm")));
      list.append (new Gdk.Pixbuf.from_file (Path.build_filename (Config.PACKAGE_PIXMAPS_DIR, "icone-48.xpm")));
      list.append (new Gdk.Pixbuf.from_file (Path.build_filename (Config.PACKAGE_PIXMAPS_DIR, "icone-64.xpm")));
      Gtk.Window.set_default_icon_list (list);
    }
    catch (Error e)
    {
      warning (e.message);
    }
  }

  public static Window get_instance ()
  {
    if (instance == null)
    {
      instance = new Window ();
    }
    return instance;
  }

  public void run ()
  {
    this.plugins = PluginManager.get_instance ();
    this.show_all ();
    this.side_panel.hide ();
    this.bottom_panel.hide ();
    Log.set_handler (null, LogLevelFlags.LEVEL_WARNING
                           | LogLevelFlags.LEVEL_MESSAGE
                           | LogLevelFlags.LEVEL_CRITICAL, this.message);
  }
}

