/* parser.vala
 *
 * Copyright (C) 2008  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 *  Nicolas Joseph <gege2061@redaction-developpez.com>
 */

public class Completion.Parser : Object
{
  private string glib_file;
  private string gobject_file;
  private List<string> vapidirs;
  private List<string> packages;
  private List<string> sources;

  public void parse_context (Vala.CodeContext context)
  {
    context.assert = false;
    context.checking = false;
    context.non_null_experimental = false;
    context.compile_only = true;

    int glib_major = 2;
    int glib_minor = 12;
    context.target_glib_major = glib_major;
    context.target_glib_minor = glib_minor;

    Vala.Parser parser = new Vala.Parser ();
    parser.parse (context);
  }

  private void analyze_context (Vala.CodeContext context)
  {
    Vala.SymbolResolver symbol_resolver = new Vala.SymbolResolver ();
    symbol_resolver.resolve (context);

    /*
    Vala.SemanticAnalyzer semantic = new Vala.SemanticAnalyzer ();
    semantic.analyze (context);
    */
  }

  private bool parse ()
  {
    Vala.CodeContext current_context = new Vala.CodeContext ();

    if (this.packages.length () != 0 || this.sources.length () != 0)
    {
      Vala.SourceFile source;

      source = new Vala.SourceFile (current_context, this.glib_file, true);
      current_context.add_source_file (source);

      source = new Vala.SourceFile (current_context, this.gobject_file, true);
      current_context.add_source_file (source);

      foreach (string item in this.packages)
      {
        if (item != null && item != this.glib_file && item != this.gobject_file)
        {
          source = new Vala.SourceFile (current_context, item, true);
          current_context.add_source_file (source);
        }
      }
      foreach (string item in this.sources)
      {
        source = new Vala.SourceFile (current_context, item, false);
        source.add_using_directive (new Vala.UsingDirective (new Vala.UnresolvedSymbol (null, "GLib", null)));
        current_context.add_source_file (source);
      }

      this.parse_context (current_context);
      this.analyze_context (current_context);
    }
    return false;
  }

  private void schedule_parse ()
  {
    Idle.add (this.parse);
  }

  private bool contains (List<string> list, string @value)
  {
    bool found = false;

    foreach (string current in list)
    {
      if (current == @value)
      {
        found = true;
        break;
      }
    }

    return found;
  }

  private string find_package (string package_name)
  {
    string filename = null;
    string found_vapidir = null;
    string result = null;

    if (!package_name.has_suffix (".vapi"))
    {
      filename = "%s.vapi".printf (package_name);
    }
    else
    {
      filename = package_name;
    }

    foreach (string vapi_dir in this.vapidirs)
    {
      string path = Path.build_filename (vapi_dir, filename);

      if (FileUtils.test (path, FileTest.EXISTS))
      {
        result = path;
        found_vapidir = vapi_dir;
        break;
      }
    }
  
    if (result != null)
    {
      //dependency check
      string dep_file = Path.build_filename (found_vapidir, package_name + ".deps");
      if (FileUtils.test (dep_file, FileTest.EXISTS))
      {
        string buffer;
        FileUtils.get_contents (dep_file, out buffer);

        foreach (string dep_name in buffer.split("\n"))
        {
          if (dep_name.length > 1)
          {
            this.add_package (dep_name);
          }
        }
      }
    }
    return result;
  }

  construct
  {
    /* @fixme Use Valide.Utils.Vala */
    this.add_vapidir ("/usr/share/vala/vapi");
    this.add_vapidir ("/usr/local/share/vala/vapi");
    this.glib_file = this.find_package ("glib-2.0");
    this.glib_file = this.find_package ("gobject-2.0");
  }

  public bool add_vapidir (string uri)
  {
    bool added = false;

    if (FileUtils.test (uri, FileTest.IS_DIR)
        && !this.contains (this.vapidirs, uri))
    {
      this.vapidirs.append (uri);
      added = true;
      this.schedule_parse ();
    }
    return added;
  }

  public bool remove_vapidir (string uri)
  {
    return false;
  }

  public bool add_package (string name)
  {
    bool added = false;
    string vapi_file = this.find_package (name);

    if (vapi_file != null)
    {
      if (!this.contains (this.packages, vapi_file))
      {
        this.packages.append (vapi_file);
        added = true;
        this.schedule_parse ();
      }
    }
    return added;
  }

  public bool remove_package (string name)
  {
    return false;
  }

  public bool add_source (string uri)
  {
    bool added = false;

    if (FileUtils.test (uri, FileTest.IS_REGULAR)
        && !this.contains (this.vapidirs, uri))
    {
      this.vapidirs.append (uri);
      added = true;
      this.schedule_parse ();
    }
    return added;
  }

  public bool remove_source (string uri)
  {
    return false;
  }

  public Result get_completions_for_name (FilterOptions options,
                                          string symbolname,
                                          string? sourcefile = null) throws Error
  {
    Result result = new Result ();
    /* @todo ... */
    return result;
  }
}

