/* manager.vala
 *
 * Copyright (C) 2008  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 *   Nicolas Joseph <gege2061@redaction-developpez.com>
 */

/*
 * This code is re-writing in the vala language of GtkSourceCompletion
 *
 * Copyright (C) 2007 - Chuchiperriman
 *
 * rev 163
 */

public class SourceCompletion.Manager : Object
{
  public const string USER_REQUEST_TRIGGER_NAME = "user-request";
  private static bool lib_initialized = false;
  private const string DEFAULT_INFO_KEYS = "<Control>i";
  private const string DEFAULT_PAGE_NEXT_KEYS = "Right";
  private const string DEFAULT_PAGE_PREV_KEYS = "Left";
  private Gtk.TextView text_view = null;
  private Popup popup;
  private List<Trigger> triggers;
  /*Providers of the triggers*/
  private HashTable<string, ProviderList> trig_prov;
  private List<Provider> providers;
  private bool active;
  private Trigger active_trigger;
  private Popup.PositionType position_type;

  private class ProviderList
  {
    public List<Provider> prov_list;
  }

  public class EventOptions
  {
    public Popup.PositionType position_type;
    public Popup.FilterType filter_type;
    public string filter_text;
    public bool autoselect;
    public bool show_bottom_bar;
  }

/* **************** GtkTextView-GscManager Control *********** */

/*
 * We save a map with a GtkTextView and his GscManager. If you 
 * call twice to gsc_proposal_new, the second time it returns
 * the previous created GscManager, not creates a new one
 */

  private static HashTable<Gtk.TextView, Manager> completion_map = null;

  private static Manager get_completion (Gtk.TextView view)
  {
    if (completion_map == null)
      completion_map = new HashTable<Gtk.TextView, Manager> (direct_hash, direct_equal);

    return completion_map.lookup (view);
  }

  private static void add_completion (Gtk.TextView view, Manager comp)
  {
    completion_map.insert (view, comp);
  }

  private static void remove_completion (Gtk.TextView view)
  {
    completion_map.remove (view);
  }

/* ********************************************************************* */

  private void end_completion ()
  {
    if (this.popup.visible)
    {
      this.popup.hide ();
    }
    else
    {
      foreach (Provider provider in this.providers)
      {
        provider.finish ();
      }
      this.active_trigger = null;
    }
  }

  [Callback]
  private void popup_proposal_select_cb (Gtk.Widget popup, Proposal proposal)
  {
    proposal.apply (this.text_view);
    this.end_completion ();
  }

  [Callback]
  private void popup_display_info_cb (Gtk.Widget popup, Proposal proposal)
  {
    string info = proposal.info;
    this.popup.set_current_info (info);
  }

  [Callback]
  private void popup_hide_cb (Gtk.Widget widget)
  {
    this.end_completion ();
  }

  [Callback]
  private bool view_focus_out_event_cb (Gtk.Widget widget, Gdk.EventFocus event)
  {
    if (this.is_visible () && ((Gtk.Widget)this.popup).has_focus)
    {
      this.end_completion ();
    }
    return false;
  }

  [Callback]
  private bool view_button_press_event_cb (Gtk.Widget widget, Gdk.EventButton event)
  {
    if (this.is_visible ())
    {
      this.end_completion ();
    }
    return false;
  }

  private void set_popup_position (EventOptions? options)
  {
    Gtk.Window parent = (Gtk.Window)this.text_view.get_ancestor (typeof (Gtk.Window));

    this.popup.set_transient_for (parent);
  
    int x, y;
  
    if (options != null) 
      this.position_type = options.position_type;
    
    switch (this.position_type)
    {
      case Popup.PositionType.CENTER_SCREEN:
        Utils.get_window_position_center_screen (this.popup, out x, out y);
      break;
      case Popup.PositionType.CENTER_PARENT:
        Utils.get_window_position_center_parent (this.popup, this.text_view, out x, out y);
      break;
      default:
        Utils.get_window_position_in_cursor (this.popup , this.text_view, out x, out y);
      break;
    }

    this.popup.move (x, y);
  }

  [Callback]
  private bool view_key_press_event_cb (Gtk.Widget view, Gdk.EventKey event)
  {
    if (this.is_visible ())
      return this.popup.manage_key (event);
    return false;
  }

  public string info_keys
  {
    owned get
    {
      return this.popup.get_key (Popup.KeysType.INFO);
    }
    set
    {
      this.popup.set_key (Popup.KeysType.INFO, value);
    }
  }

  public string next_page_keys
  {
    owned get
    {
      return this.popup.get_key (Popup.KeysType.PAGE_NEXT);
    }
    set
    {
      this.popup.set_key (Popup.KeysType.PAGE_NEXT, value);
    }
  }

  public string previous_page_keys
  {
    owned get
    {
      return this.popup.get_key (Popup.KeysType.PAGE_PREV);
    }
    set
    {
      this.popup.set_key (Popup.KeysType.PAGE_PREV, value);
    }
  }

  construct
  {
    if (!Manager.lib_initialized)
    {
      Manager.lib_initialized = true;
    }
  
    this.active = false;
    this.providers = null;
    this.triggers = null;
    this.popup = null;
    this.active_trigger = null;
    this.position_type = Popup.PositionType.CURSOR;
    this.trig_prov = new HashTable<string, ProviderList> (str_hash, str_equal);

    this.popup = new Popup ();
  
    this.popup.proposal_selected += popup_proposal_select_cb;
    this.popup.display_info += popup_display_info_cb;
    this.popup.hide += popup_hide_cb;
  }

  public override void finalize ()
  {
    if (this.active)
    {
      this.deactivate ();
    }
    this.remove_completion (this.text_view);
  }

  private Manager ()
  {
  }

  public static Manager get_instance (Gtk.TextView view)
  {
    Manager completion = Manager.get_completion (view);
    if (completion != null)
    {
      return completion;
    }

    completion = new Manager ();
    completion.text_view = view;

    Manager.add_completion (view, completion);

    return completion;
  }

  public bool register_provider (Provider provider, string trigger_name)
  {
    Trigger trigger = this.get_trigger (trigger_name);
    if (trigger == null)
      return false;
    ProviderList? pl = this.trig_prov.lookup (trigger_name);
    assert (pl != null);
    pl.prov_list.append (provider);
    Provider prov = this.get_provider(provider.get_name ());
    if (prov != null)
      return false;
    this.providers.append (provider);
    
    return true;
  }

  public bool unregister_provider (Provider provider, string trigger_name)
  {
    if (this.providers.find (provider) == null)
      return false;
    Trigger trigger = this.get_trigger (trigger_name);
    if (trigger == null)
      return false;
    ProviderList? pl = this.trig_prov.lookup (trigger_name);
    assert (pl != null);
    pl.prov_list.remove (provider);

    this.providers.remove (provider);
    return true;
  }

  public Gtk.TextView get_view ()
  {
    return this.text_view;
  }

  public void update_event_options (EventOptions? options)
  {
    if (options != null)
    {
      this.set_popup_position (options);

      this.popup.set_filter_type (options.filter_type);

      this.popup.bottom_bar_set_visible (options.show_bottom_bar);

      if (options.filter_text != null)
        this.popup.set_filter_text (options.filter_text);
    }
  }

  public void trigger_event_with_opts (string trigger_name,
                                       EventOptions? options,
                                       void* event_data)
  {
    List<Proposal> data_list;
    List<Proposal> final_list = null;

    Trigger trigger = this.get_trigger (trigger_name);
    return_if_fail (trigger != null);

    if (this.text_view.has_focus)
    {
      this.popup.clear ();
    
      ProviderList? pl = this.trig_prov.lookup (trigger_name);
      if (pl != null)
      {
        /*providers_list = this.providers;*/      
        if (pl.prov_list != null)
        {
          /*Getting the data...*/
          foreach (Provider provider in pl.prov_list)
          {
            data_list = provider.get_proposals (trigger);
            foreach (Proposal proposal in data_list)
            {
              final_list.append (proposal);
            }
          }

          if (final_list != null)
          {
            /* If there are not proposals, we don't show the popup */
            if (final_list.length () > 0)
            {
              /* Insert the data into the model */
              foreach (Proposal last_proposal in final_list)
              {
                this.popup.add_data (last_proposal);
              }
              if (this.text_view.has_focus)
              {
                if (options == null)
                {
                  EventOptions opts = new EventOptions ();
                  opts.position_type = Popup.PositionType.CURSOR;
                  opts.filter_type = Popup.FilterType.NONE;
                  opts.filter_text = null;
                  opts.autoselect = false;
                  opts.show_bottom_bar = true;

                  this.update_event_options (opts);
                }
                else
                {
                  this.update_event_options (options);
                }
                this.popup.show_or_update ();
                this.active_trigger = trigger;
              }
            }
            else if (this.popup.visible)
            {
              this.end_completion ();
            }
          }
          else if (this.is_visible ())
          {
            this.end_completion ();
          }
        }
        else if (this.is_visible ())
        {
          this.end_completion ();
        }
      }
    }
  }

  public void trigger_event (string trigger_name, void* event_data)
  {
    this.trigger_event_with_opts (trigger_name, null, event_data);
  }

  public bool is_visible ()
  {
    return this.popup.visible;
  }

  public static Manager get_from_view (Gtk.TextView view)
  {
    return Manager.get_completion (view);
  }

  public Provider? get_provider (string provider_name)
  {
    weak List<Provider> plist = this.providers;
    if (plist != null)
    {
      foreach (Provider provider in plist)
      {
        if (provider.get_name () == provider_name)
        {
          return provider;
        }
      }
    }

    return null;
  }

  public void register_trigger (Trigger trigger)
  {
    string trigger_name = trigger.get_name ();
    ProviderList pl = this.trig_prov.lookup (trigger_name);
    /*Only register the trigger if it has not been registered yet*/
    if (pl == null)
    {
      this.triggers.append (trigger);
      string tn = trigger.get_name ();
      this.trig_prov.insert (tn, new ProviderList ());
      if (this.active)
      {
        trigger.activate ();
      }
    }
  }

  public void unregister_trigger (Trigger trigger)
  {
    if (this.triggers.find (trigger) == null)
      return;
    this.triggers.remove (trigger);
    if (this.active)
    {
      trigger.deactivate ();
    }
    this.trig_prov.remove (trigger.get_name ());
  }

  public Trigger? get_trigger (string trigger_name)
  {
    foreach (Trigger trigger in this.triggers)
    {
      if (trigger.get_name () == trigger_name)
      {
        return trigger;
      }
    }
    return null;
  }

  public void activate ()
  {
    this.text_view.key_press_event += view_key_press_event_cb;
    this.text_view.focus_out_event += view_focus_out_event_cb;
    this.text_view.button_press_event += view_button_press_event_cb;

    /* We activate the triggers*/
    foreach (Trigger trigger in this.triggers)
    {
      trigger.activate ();
    }

    this.active = true;
  }

  public void deactivate ()
  {
    this.text_view.key_press_event -= view_key_press_event_cb;
    this.text_view.focus_out_event -= view_focus_out_event_cb;
    this.text_view.button_press_event -= view_button_press_event_cb;

    foreach (Trigger trigger in this.triggers)
    {
      trigger.deactivate ();
    }
  
    this.active = false;
  }

  public void finish_completion ()
  {
    if (this.is_visible ())
    {
      this.end_completion ();
    }
  }

  public Trigger get_active_trigger ()
  {
    return this.active_trigger;
  }

  public void set_current_info (string info)
  {
    this.popup.set_current_info (info);
  }

  public void get_current_event_options (out EventOptions options)
  {
    options.filter_type = this.popup.get_filter_type ();
    options.position_type = this.position_type;
    options.filter_text = this.popup.get_filter_text ();
    options.autoselect = false;
    options.show_bottom_bar = this.popup.bottom_bar_get_visible ();
  }
}

