/* file-browser.vala
 *
 * Copyright (C) 2009 Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <gege2061@redaction-developpez.com>
 */

using Valide;

public class FileBrowser : Plugin
{
  /* enum for tree model columns, not all is currently used */
  enum Col
  {
    ICON,
    NAME,
    PATH,
    NB
  }

  private Gtk.VBox box;
  private Gtk.TreeView tree_view;
  private string _cwd;

  private string cwd
  {
    get
    {
      return this._cwd;
    }
    set
    {
      this._cwd = value;
      this.populate (this, this);
    }
  }

  private bool populate_asc ()
  {
    Gtk.ListStore model = (Gtk.ListStore)this.tree_view.get_model ();
    model.clear ();

    try
    {
      Dir dir = Dir.open (this.cwd);
      unowned string filename;
      string path;
      Gtk.TreeIter iter;
      Gdk.Pixbuf pixbuf;

      pixbuf = Utils.get_pixbuf_for_stock (Gtk.STOCK_DIRECTORY, Gtk.IconSize.BUTTON);
      model.append (out iter);
      model.set (iter, Col.ICON, pixbuf, Col.NAME, "..",
                 Col.PATH, Path.get_dirname (this.cwd));
      while ((filename = dir.read_name ()) != null)
      {
        if (!filename.has_prefix ("."))
        {
          path = Path.build_filename (this.cwd, filename);
          if (FileUtils.test (path, FileTest.IS_DIR))
          {
            pixbuf = Utils.get_pixbuf_for_stock (Gtk.STOCK_DIRECTORY, Gtk.IconSize.BUTTON);
          }
          else
          {
            pixbuf = Utils.get_pixbuf_for_stock (Gtk.STOCK_FILE, Gtk.IconSize.BUTTON);
          }
          model.append (out iter);
          model.set (iter, Col.ICON, pixbuf, Col.NAME, filename, Col.PATH, path);
        }
      }
    }
    catch (Error e)
    {
      warning (e.message);
    }
    return false;
  }

  /**
   * Call to re-populate tree view.
   */
  [Callback]
  private static void populate (Object sender, FileBrowser self)
  {
    Idle.add (self.populate_asc);
  }

  [Callback]
  private void on_row_activated (Gtk.TreeView sender, Gtk.TreePath tree_path,
                                 Gtk.TreeViewColumn column)
  {
    string path;
    Gtk.TreeIter iter;
        
    Gtk.TreeModel model = this.tree_view.get_model ();
    model.get_iter (out iter, tree_path);

    model.get (iter, Col.PATH, out path);

    if (FileUtils.test (path, FileTest.IS_DIR))
    {
      this.cwd = path;
      this.populate (this, this);
    }
    else
    {
      try
      {
        if (ProjectManager.get_instance ().accept_file (path))
        {
          ProjectManager.get_instance ().open (path);
        }
        else
        {
          DocumentManager.get_instance ().create (path);
        }
      }
      catch (Error e)
      {
        warning (e.message);
      }
    }
  }

  private int sort_func (Gtk.TreeModel model, Gtk.TreeIter a, Gtk.TreeIter b)
  {
    int ret = 0;
    string path_a, path_b;
    string name_a, name_b;
    bool a_is_dir, b_is_dir;

    model.get (a, Col.NAME, out name_a, Col.PATH, out path_a);
    model.get (b, Col.NAME, out name_b, Col.PATH, out path_b);

    a_is_dir = FileUtils.test (path_a, FileTest.IS_DIR);
    b_is_dir = FileUtils.test (path_b, FileTest.IS_DIR);

    if (a_is_dir && !b_is_dir)
    {
      ret = -1;
    }
    else if (!a_is_dir && b_is_dir)
    {
      ret = 1;
    }
    else
    {
      ret = strcmp (name_a, name_b);
    }
    return ret;
  }

  construct
  {
    this.box = new Gtk.VBox (false, 0);

    /* create some nice padding */
    this.box.set_spacing (2);
    this.box.set_border_width (2);
 
    /* create and pack tree view */
    Gtk.ScrolledWindow scrolled = new Gtk.ScrolledWindow (null, null);
    scrolled.set_policy (Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC);
    scrolled.set_shadow_type (Gtk.ShadowType.IN);
    this.box.pack_start (scrolled, true, true, 0);

    this.tree_view = new Gtk.TreeView ();
    this.tree_view.set_rules_hint (false);
    this.tree_view.set_headers_visible (false);
    scrolled.add (this.tree_view);

    /* create model */
    Gtk.ListStore store = new Gtk.ListStore (Col.NB,
                                             typeof (Gdk.Pixbuf),
                                             typeof (string),
                                             typeof (string));
    store.set_sort_column_id (Col.NAME, Gtk.SortType.ASCENDING);
    store.set_sort_func (Col.NAME, this.sort_func);
    /* main column */
    Gtk.TreeViewColumn col = new Gtk.TreeViewColumn ();
    col.set_title(_("File"));
    {
      Gtk.CellRendererPixbuf renderer = new Gtk.CellRendererPixbuf ();
      col.pack_start (renderer, false);
      col.set_attributes (renderer, "pixbuf", Col.ICON);
    }
    {
      Gtk.CellRendererText renderer = new Gtk.CellRendererText ();
      col.pack_start (renderer, false);
      col.set_attributes (renderer, "markup", Col.NAME);
    }
    this.tree_view.append_column (col);
    this.tree_view.set_model (store);

    this.cwd = Environment.get_home_dir ();

    /* attach signal callbacks for tree view */
    this.tree_view.row_activated += on_row_activated;

    Window.get_instance ().side_panel.add_item_with_stock_icon (this.box,
                                                                _("File Browser"),
                                                                Gtk.STOCK_DIRECTORY);

    this.box.show_all ();
  }

  ~FileBrowser ()
  {
    Window.get_instance ().side_panel.remove_item (this.box);
  }
}

[ModuleInit]
public Type register_plugin ()
{
  return typeof (FileBrowser);
}

