/* todo-manager.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <gege2061@redaction-developpez.com>
 */

using Valide;

private class TodoItem
{
  public string type;
  public string text;
  public string file;
  public int line;
  public weak AbstractTextDocument document;
}

/**
 * @todo Add support for author, like C::B
 */
public class Todo : Plugin
{
  enum Col
  {
    TYPE,
    TEXT,
    FILE,
    LINE,
    DOCUMENT,
    NB
  }

  private Gtk.ScrolledWindow scrolled_window;
  private Gtk.TreeView tree_view;
  private DocumentManager documents_;
  private uint status_context;

  private bool only_load_active_tab
  {
    get
    {
      return true;
    }
  }

  private DocumentManager documents
  {
    get
    {
      return this.documents_;
    }
    set
    {
      this.documents_ = value;

      this.status_context = Statusbar.get_instance ().get_context_id ("valide_todo_list");

      Signal.connect_after (this.documents, "tab-removed", (Callback)populate_list, this);
      Signal.connect_after (this.documents, "switch-page", (Callback)on_active_tab_changed, this);
      Signal.connect_after (this.documents, "tab-saved", (Callback)populate_list, this);
    }
  }

  [Callback]
  private void on_row_activated (Gtk.TreeView sender, Gtk.TreePath path,
                                 Gtk.TreeViewColumn column)
  {
    Gtk.TreeIter iter;        
    Gtk.TreeModel model = this.tree_view.get_model ();
    AbstractTextDocument document;
    int line;

    model.get_iter (out iter, path);
    model.get (iter, Col.LINE, out line, Col.DOCUMENT, out document, -1);

    DocumentManager.get_instance ().current = document;
    document.goto_line (line - 1);
    document.grab_focus ();
  }

  [Callback]
  private static void populate_list (Object sender, Todo self)
  {
    self.tree_view.sensitive = false;
    /* clear data */
    Gtk.ListStore model = (Gtk.ListStore)self.tree_view.get_model ();
    model.clear ();

    if (self.documents.current != null)
    {
      int count = 0;
      /* find out if we want to load all symbols or just active tab */
      List<iDocument> doc = null;
      if (self.only_load_active_tab)
      {
        doc.prepend (self.documents.current);
      }
      else
      {
        doc = self.documents.documents.copy ();
      }

      foreach (iDocument document in doc)
      {
        if (document is AbstractTextDocument)
        {
          List<TodoItem> items = self.parse ((AbstractTextDocument)document);
          if (items != null)
          {
            foreach (TodoItem item in items)
            {
              self.add_line (item);
            }
          }
          count++;
        }
      }
      if (count > 0)
      {
        self.tree_view.sensitive = true;
      }
    }
  }

  [Callback]
  private static void on_active_tab_changed (DocumentManager sender,
                                             Gtk.NotebookPage page,
                                             uint page_num, Todo self)
  {
    /* only needed in single-file mode  */
    if (self.only_load_active_tab)
    {
      populate_list (sender, self);
    }
  }

  private void add_line (TodoItem item)
  {
    Gtk.TreeIter iter;
    Gtk.ListStore list_store = (Gtk.ListStore)this.tree_view.get_model ();

    list_store.append (out iter);
    list_store.set (iter, Col.TYPE, item.type, Col.TEXT, item.text,
                          Col.FILE, item.file, Col.LINE, item.line,
                          Col.DOCUMENT, item.document);
  }

  private List<TodoItem> parse (AbstractTextDocument d)
  {
    List<TodoItem> items = null;
    Gtk.TextBuffer buffer = d.buffer;
    Gtk.TextIter start, end;

    buffer.get_start_iter (out start);
    buffer.get_end_iter (out end);
    string text = buffer.get_text (start, end, false);
    string[] lines = text.split ("\n");

    try
    {
      Regex regex = new Regex ("@(TODO|FIXME|CHANGED|NOTE|OPTIMIZE|IMPROVE|BUG)[:]?[\\s]*(.*)",
                               RegexCompileFlags.CASELESS);
    
      for (int i = 0; lines[i] != null; i++)
      {
        try
        {
          MatchInfo match_info;
          if (regex.match_full (lines[i], -1, 0, 0, out match_info))
          {
            while (match_info.matches ())
            {
              TodoItem item = new TodoItem ();

              item.type = match_info.fetch (1).up ();
              item.text = match_info.fetch (2);
              if (item.text.has_suffix ("*/"))
              {
                item.text = item.text.substring (0, item.text.len () - 2);
              }
              item.document = d;
              item.file = d.filename;
              item.line = i+1;
              items.append (#item);
              match_info.next ();
            }
          }
        }
        catch (Error e)
        {
          warning (e.message);
        }
      }
    }
    catch (Error e)
    {
      debug (e.message);
    }
    return items;
  }

  construct
  {
    this.scrolled_window = new Gtk.ScrolledWindow (null, null);

    this.scrolled_window.set_policy (Gtk.PolicyType.AUTOMATIC,
                                     Gtk.PolicyType.AUTOMATIC);

    Gtk.ListStore list_store = new Gtk.ListStore (Col.NB,
                                                  typeof (string), typeof (string),
                                                  typeof (string), typeof (int),
                                                  typeof (Object));
    this.tree_view = new Gtk.TreeView.with_model (list_store);
    this.tree_view.row_activated += this.on_row_activated;
    this.scrolled_window.add (this.tree_view);

    Gtk.CellRendererText render = new Gtk.CellRendererText ();
    Gtk.TreeViewColumn column = new Gtk.TreeViewColumn.with_attributes (_("Type"), render, "text", Col.TYPE, null);
    this.tree_view.append_column (column);
    render = new Gtk.CellRendererText ();
    column = new Gtk.TreeViewColumn.with_attributes (_("Text"), render, "text", Col.TEXT, null);
    this.tree_view.append_column (column);
    render = new Gtk.CellRendererText ();
    column = new Gtk.TreeViewColumn.with_attributes (_("File"), render, "text", Col.FILE, null);
    this.tree_view.append_column (column);
    render = new Gtk.CellRendererText ();
    column = new Gtk.TreeViewColumn.with_attributes (_("Line"), render, "text", Col.LINE, null);
    this.tree_view.append_column (column);

    this.documents = DocumentManager.get_instance ();
    this.tree_view.sensitive = false;
    Window.get_instance ().bottom_panel.add_item_with_stock_icon (this.scrolled_window,
                                                                  "Todo list", Gtk.STOCK_EDIT);
    this.scrolled_window.show_all ();
  }

  ~Todo ()
  {
    SignalHandler.disconnect_by_func (this.documents, (void*)populate_list, this);
    SignalHandler.disconnect_by_func (this.documents, (void*)on_active_tab_changed, this);
    this.tree_view.row_activated -= this.on_row_activated;
    Window.get_instance ().bottom_panel.remove_item (this.scrolled_window);
  }
}

[ModuleInit]
public Type register_plugin ()
{
  return typeof (Todo);
}

